@@ -1,6 +1,7 @@
 from swibots.api.auth.controllers import UserController
 from swibots.base import SwitchRestClient
-from swibots.config import APP_CONFIG
+from swibots.config import get_config
+import swibots
 
 
 class AuthClient(SwitchRestClient):
@@ -12,8 +13,10 @@ class AuthClient(SwitchRestClient):
         - :attr:`users`: :obj:`~switch.api.auth.controllers.UserController` : The users controller
     """
 
-    def __init__(self, base_url: str = APP_CONFIG["AUTH_SERVICE"]["BASE_URL"]):
-        super().__init__(base_url)
+    def __init__(self, app: "swibots.App", base_url: str = None):
+        """Initialize the auth client"""
+        base_url = base_url or get_config()["AUTH_SERVICE"]["BASE_URL"]
+        super().__init__(app, base_url)
         self._users = None
         self._authorization = None
 
@@ -30,6 +33,6 @@ class AuthClient(SwitchRestClient):
 
     def prepare_request_headers(self, headers: dict) -> dict:
         headers = super().prepare_request_headers(headers)
-        if self._auth_token is not None:
-            headers["authtoken"] = f"{self._auth_token}"
+        if self.token is not None:
+            headers["authtoken"] = f"{self.token}"
         return headers
