@@ -1,7 +1,9 @@
+import asyncio
 import json
 import logging
-from typing import TYPE_CHECKING, List
-from swibots.api.chat.models import Message, GroupChatHistory
+from typing import TYPE_CHECKING, List, Optional
+from swibots.api.common.models import User, MediaUploadRequest
+from swibots.api.chat.models import Message, GroupChatHistory, InlineMarkup, InlineQuery, InlineQueryAnswer
 from swibots.error import SwitchError
 from swibots.utils.types import JSONDict
 from swibots.api.community.models import Channel, Community, Group
@@ -9,7 +11,7 @@ from swibots.api.community.models import Channel, Community, Group
 if TYPE_CHECKING:
     from swibots.api.chat import ChatClient
 
-_logger = logging.getLogger(__name__)
+log = logging.getLogger(__name__)
 
 BASE_PATH = "/v1/message"
 
@@ -24,6 +26,25 @@ class MessageController:
     def __init__(self, client: "ChatClient"):
         self.client = client
 
+    async def new_message(self, to: Optional[int | User] = None, channel: Optional[Channel | str] = None, group: Optional[Group | str] = None) -> Message:
+        """Create a new message"""
+        if isinstance(to, User):
+            to = to.id
+
+        if isinstance(channel, Channel):
+            channel = channel.id
+
+        if isinstance(group, Group):
+            group = group.id
+
+        return Message(
+            user_id=self.client.user.id,
+            receiver_id=to,
+            channel_id=channel,
+            group_id=group,
+            app=self.client.app,
+        )
+
     async def get_messages(self, user_id: int = None) -> List[Message]:
         """Get messages for a user
 
@@ -38,15 +59,16 @@ class MessageController:
         """
         if user_id is None:
             user_id = self.client.user.id
-        _logger.debug("Getting messages for user %s", user_id)
+        log.debug("Getting messages for user %s", user_id)
         response = await self.client.get(f"{BASE_PATH}/{user_id}")
-        return Message.build_from_json_list(response.data)
+        return self.client.build_list(Message, response.data)
 
-    async def send_message(self, message: Message) -> Message:
+    async def send_message(self, message: Message, media: MediaUploadRequest = None) -> Message:
         """Send a message
 
         Parameters:
             message (``~switch.api.chat.models.Message``): The message to send
+            media (``~switch.api.common.models.MediaUploadRequest``, *optional*): The media to send with the message
 
         Returns:
             ``~switch.api.chat.models.Message``: The message
@@ -55,9 +77,73 @@ class MessageController:
             ``~switch.error.SwitchError``: If the message could not be sent
         """
         data = message.to_json_request()
-        _logger.debug("Sending message %s", json.dumps(data))
-        response = await self.client.post(f"{BASE_PATH}/create", data=data)
-        return Message.build_from_json(response.data["message"])
+        log.debug("Sending message %s", json.dumps(data))
+
+        if media:
+            url = f"{BASE_PATH}/create-with-media"
+            data.update(media.data_to_request())
+            upload_fn = self.client.post(
+                url, form_data=data, files=media.file_to_request(url))
+            if media.block:
+                response = await upload_fn
+            else:
+                asyncio.get_event_loop().create_task(upload_fn)
+                return
+        else:
+            response = await self.client.post(f"{BASE_PATH}/create", data=data)
+        return self.client.build_object(Message, response.data["message"])
+
+    async def send_text(self, text: str, to: Optional[int | User] = None, channel: Optional[Channel | str] = None, group: Optional[Group | str] = None,  inline_markup: InlineMarkup = None, media: MediaUploadRequest = None) -> Message:
+        """Send a message with text
+
+        Parameters:
+            to (``int`` | ``~switch.api.common.models.User``, *optional*): The user id to send the message to. Defaults to the current user id.
+            text (``str``): The text to send
+            channel (``~switch.api.community.models.Channel``, *optional*): The channel to send the message to
+            group (``~switch.api.community.models.Group``, *optional*): The group to send the message to
+            inline_markup (``~switch.api.chat.models.InlineMarkup``, *optional*): The inline markup to send with the message
+            media (``~switch.api.common.models.MediaUploadRequest``, *optional*): The media to send with the message
+
+        Returns:
+            ``~switch.api.chat.models.Message``: The message
+
+        Raises:
+            ``~switch.error.SwitchError``: If the message could not be sent
+        """
+        message = await self.new_message(to, channel, group)
+        message.message = text
+        message.inline_markup = inline_markup
+        return await self.send_message(message, media)
+
+    async def reply(self, message: int | Message, reply: Message, media: MediaUploadRequest = None) -> Message:
+        if isinstance(message, Message):
+            id = message.id
+        else:
+            id = message
+        reply.replied_to_id = id
+        return await self.send_message(reply, media)
+
+    async def reply_text(self, message: int | Message, text: str, inline_markup: InlineMarkup = None, media: MediaUploadRequest = None) -> Message:
+        """Reply to a message with text
+
+        Parameters:
+            message (``~switch.api.chat.models.Message``): The message to reply to
+            text (``str``): The text to reply with
+
+        Returns:
+            ``~switch.api.chat.models.Message``: The message
+
+        Raises:
+            ``~switch.error.SwitchError``: If the message could not be sent
+        """
+        if isinstance(message, Message):
+            id = message.id
+        else:
+            id = message
+
+        m = Message(message=text, inline_markup=inline_markup)
+
+        return await self.reply(id, m, media)
 
     async def edit_message(self, message: Message) -> Message:
         """Edit a message
@@ -72,10 +158,29 @@ class MessageController:
             ``~switch.error.SwitchError``: If the message could not be edited
         """
         data = message.to_json_request()
-        _logger.debug("Editing message %s", json.dumps(data))
+        log.debug("Editing message %s", json.dumps(data))
         response = await self.client.put(f"{BASE_PATH}?id={message.id}", data=data)
         return message
 
+    async def edit_message_text(self, message: int | Message, text: str, inline_markup: InlineMarkup = None) -> Message:
+        """Edit a message with text
+
+        Parameters:
+            message (``~switch.api.chat.models.Message``): The message to edit
+            text (``str``): The text to edit with
+
+        Returns:
+            ``~switch.api.chat.models.Message``: The message
+
+        Raises:
+            ``~switch.error.SwitchError``: If the message could not be edited
+        """
+        if isinstance(message, Message):
+            id = message.id
+        else:
+            id = message
+        return await self.edit_message(Message(id=id, message=text, inline_markup=inline_markup))
+
     async def delete_message(self, message: int | Message) -> bool:
         """Delete a message
 
@@ -92,7 +197,7 @@ class MessageController:
             id = message.id
         else:
             id = message
-        _logger.debug("Deleting message %s", id)
+        log.debug("Deleting message %s", id)
         response = await self.client.delete(f"{BASE_PATH}/{id}")
         return True
 
@@ -110,7 +215,7 @@ class MessageController:
             ``~switch.error.SwitchError``: If the messages could not be deleted
 
         """
-        _logger.debug("Deleting messages for user %s", recipient_id)
+        log.debug("Deleting messages for user %s", recipient_id)
         if user_id is None:
             user_id = self.client.user.id
 
@@ -145,9 +250,9 @@ class MessageController:
         if user_id is None:
             user_id = self.client.user.id
 
-        _logger.debug("Getting messages for user %s", recipient_id)
+        log.debug("Getting messages for user %s", recipient_id)
         response = await self.client.get(f"{BASE_PATH}/{user_id}/{recipient_id}?{str_q}")
-        return Message.build_from_json_list(response.data)
+        return self.client.build_list(Message, response.data["messages"])
 
     async def forward_message(
         self,
@@ -188,15 +293,16 @@ class MessageController:
 
         strQuery = q.join("&")
 
-        _logger.debug("Forwarding message %s", id)
+        log.debug("Forwarding message %s", id)
         response = await self.client.post(f"{BASE_PATH}/forward/{id}?{strQuery}")
-        return Message.build_from_json(response.data["message"])
+        return self.client.build_object(Message, response.data["message"])
+        # return Message.build_from_json(response.data["message"])
 
     async def get_message(self, message: int | Message) -> Message:
         """Get a message by id
 
         Parameters:
-            message (``int`` | ``~switch.api.chat.models.Message``): The message id or message to get
+            message (``int`` | ``~switch.api.chat.models.Message``): The message id or message to get, if a message is passed, the id will be extracted from it.
 
         Returns:
             ``~switch.api.chat.models.Message``: The message
@@ -208,9 +314,10 @@ class MessageController:
             id = message.id
         else:
             id = message
-        _logger.debug("Getting message %s", id)
+        log.debug("Getting message %s", id)
         response = await self.client.get(f"{BASE_PATH}/{id}")
-        return Message.build_from_json(response.data["message"])
+        return self.client.build_object(Message, response.data["message"])
+        # return Message.build_from_json(response.data["message"])
 
     async def get_group_chat_history(
         self,
@@ -236,7 +343,7 @@ class MessageController:
             ``~switch.error.SwitchError``: If the group chat history could not be retrieved
 
         """
-        _logger.debug("Getting group chat history for group %s", group_id)
+        log.debug("Getting group chat history for group %s", group_id)
         q = ["isChannel=false"]
         if page_limit:
             q.append(f"pageLimit={page_limit}")
@@ -251,7 +358,8 @@ class MessageController:
             user_id = self.client.user.id
 
         response = await self.client.get(f"{BASE_PATH}/group/{user_id}/{group_id}?{str_q}")
-        return GroupChatHistory.build_from_json(response.data)
+        return self.client.build_object(GroupChatHistory, response.data)
+        # return GroupChatHistory.build_from_json(response.data)
 
     async def get_channel_chat_history(
         self,
@@ -277,7 +385,7 @@ class MessageController:
             ``~switch.error.SwitchError``: If the channel chat history could not be retrieved
 
         """
-        _logger.debug("Getting channel chat history for channel %s", channel_id)
+        log.debug("Getting channel chat history for channel %s", channel_id)
         q = ["isChannel=true"]
         if page_limit:
             q.append(f"pageLimit={page_limit}")
@@ -292,7 +400,8 @@ class MessageController:
             user_id = self.client.user.id
 
         response = await self.client.get(f"{BASE_PATH}/group/{user_id}/{channel_id}?{str_q}")
-        return GroupChatHistory.build_from_json(response.data)
+        return self.client.build_object(GroupChatHistory, response.data)
+        # return GroupChatHistory.build_from_json(response.data)
 
     async def get_community_media_files(self, community_id: str) -> List[Message]:
         """Get community media files
@@ -306,9 +415,10 @@ class MessageController:
         Raises:
             ``~switch.error.SwitchError``: If the community media files could not be retrieved
         """
-        _logger.debug("Getting community media files for community %s", community_id)
+        log.debug("Getting community media files for community %s", community_id)
         response = await self.client.get(f"{BASE_PATH}/media?communityId={community_id}")
-        return Message.build_from_json_list(response.data)
+        return self.client.build_list(Message, response.data)
+        # return Message.build_from_json_list(response.data)
 
     async def get_community_media_files_by_status(
         self, community_id: str, status: str
@@ -326,11 +436,12 @@ class MessageController:
         Raises:
             ``~switch.error.SwitchError``: If the community media files could not be retrieved
         """
-        _logger.debug("Getting community media files for community %s", community_id)
+        log.debug("Getting community media files for community %s", community_id)
         response = await self.client.get(
             f"{BASE_PATH}/media?communityId={community_id}&status={status}"
         )
-        return Message.build_from_json_list(response.data)
+        return self.client.build_list(Message, response.data)
+        # return Message.build_from_json_list(response.data)
 
     async def get_user_media_files(self, user_id: int = None) -> List[Message]:
         """Get user media files
@@ -347,9 +458,10 @@ class MessageController:
         """
         if user_id is None:
             user_id = self.client.user.id
-        _logger.debug("Getting user media files for user %s", user_id)
+        log.debug("Getting user media files for user %s", user_id)
         response = await self.client.get(f"{BASE_PATH}/media/{user_id}")
-        return Message.build_from_json_list(response.data)
+        return self.client.build_list(Message, response.data)
+        # return Message.build_from_json_list(response.data)
 
     async def clear_conversation(self, receiver_id: int) -> bool:
         """Clear a conversation
@@ -363,7 +475,7 @@ class MessageController:
         Raises:
             ``~switch.error.SwitchError``: If the conversation could not be cleared
         """
-        _logger.debug("Clearing conversation %s", receiver_id)
+        log.debug("Clearing conversation %s", receiver_id)
         response = await self.client.get(f"{BASE_PATH}/clearconversationwith/{receiver_id}")
         return True
 
@@ -382,9 +494,10 @@ class MessageController:
         if user_id is None:
             user_id = self.client.user.id
 
-        _logger.debug("Get flag messages for %s", user_id)
+        log.debug("Get flag messages for %s", user_id)
         response = await self.client.get(f"{BASE_PATH}/flag?userId={user_id}")
-        return Message.build_from_json_list(response.data)
+        return self.client.build_list(Message, response.data)
+        # return Message.build_from_json_list(response.data)
 
     async def flag_message(self, message: Message | int) -> bool:
         """Flag a message
@@ -402,7 +515,7 @@ class MessageController:
             message_id = message.id
         else:
             message_id = message
-        _logger.debug("Flagging message %s", message_id)
+        log.debug("Flagging message %s", message_id)
         response = await self.client.post(f"{BASE_PATH}/flag?messageId={message_id}")
         return True
 
@@ -418,6 +531,34 @@ class MessageController:
         if user_id is None:
             user_id = self.client.user.id
 
-        _logger.debug("Get unread messages count for %s", user_id)
+        log.debug("Get unread messages count for %s", user_id)
         response = await self.client.get(f"{BASE_PATH}/unread-messages?userId={user_id}")
         return response.data
+
+    async def answer_inline_query(self, query: InlineQuery, answer: InlineQueryAnswer) -> bool:
+        """Answer an inline query
+
+        Parameters:
+            query (``~switch.api.chat.models.InlineQuery``): The inline query
+            answer (``~switch.api.chat.models.InlineQueryAnser``): The answer
+
+        Returns:
+            ``bool``: True if the query was answered
+
+        Raises:
+            ``~switch.error.SwitchError``: If the query could not be answered
+        """
+        if not isinstance(query, InlineQuery):
+            raise TypeError("query must be an InlineQuery instance")
+
+        if isinstance(answer, str):
+            answer = InlineQueryAnswer(query_id=query.query_id, title=answer, results=[
+            ], cache_time=0, is_personal=True, next_offset=None, pm_text=None, pm_parameter=None, user_id=query.user_id)
+
+        if isinstance(answer, List):
+            answer = InlineQueryAnswer(query_id=query.query_id, title=None, results=answer, cache_time=0,
+                                       is_personal=True, next_offset=None, pm_text=None, pm_parameter=None, user_id=query.user_id)
+
+        log.debug("Answering inline query %s", query.query_id)
+        response = await self.client.post(f"{BASE_PATH}/inline/answer", answer.to_json_request())
+        return response.data
