@@ -2,5 +2,7 @@ from .chat_event import ChatEvent
 from .message_event import MessageEvent
 from .command_event import CommandEvent
 from .callback_query_event import CallbackQueryEvent
+from .inline_query_event import InlineQueryEvent
 
-__all__ = ["ChatEvent", "MessageEvent", "CommandEvent", "CallbackQueryEvent"]
+__all__ = ["ChatEvent", "MessageEvent", "CommandEvent",
+           "CallbackQueryEvent", "InlineQueryEvent"]
